/***************************************************************************
 *   Copyright (C) 2003 by Dan Muller                                      *
 *   dan at verliba dot cz                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "cserverdc.h"
#include "ctrigger.h"
#include "cconndc.h"
#include "curr_date_time.h"
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


namespace nDirectConnect {
namespace nTables {
/// Constructor
cTrigger::cTrigger()
{
	mFlags = 0;
	mMinClass=0;
	mMaxClass=10;
	mMaxLines=0;
	mMaxSize=0;
	mDelayUser=0;
	mDelayTotal=0;
}
/// Destructor
cTrigger::~cTrigger(){}

/** \fn cTrigger::DoIt(istringstream &cmd_line, cConnDC *conn, cServerDC &server)
	\brief Send the defined file contents to the user connection
	\param conn the connecten that trigger the trigger
	\param server reference for the DC server
	\param cmd_line what user has writter
*/
int cTrigger::DoIt(istringstream &cmd_line, cConnDC *conn, cServerDC &server)
{
	string buf, filename;
	if (conn && conn->mpUser)
	{
		int uclass = conn->mpUser->mClass;
		if ((uclass >= this->mMinClass) &&(uclass <= this->mMaxClass)) {
			if (mFlags & eTF_DB) {
				buf = mDefinition;
			} else {
				ReplaceVarInString(mDefinition,"CFG", filename, server.mConfigBaseDir);
				ReplaceVarInString(filename,"CC", filename, conn->mCC);
				if (!LoadFileInString(filename,buf)) return 0;
			}
			if (mFlags & eTF_VARS) {
				cTime theTime(server.mTime);
				theTime -= server.mStartTime;			
				ReplaceVarInString(buf, "CC", buf, conn->mCC);
				ReplaceVarInString(buf, "IP", buf, conn->AddrIP());
				ReplaceVarInString(buf, "HOST", buf, conn->AddrHost());
				ReplaceVarInString(buf, "USERS", buf, (int)server.mUserList.size());
				ReplaceVarInString(buf, "UPTIME", buf, theTime.AsPeriod().AsString());
				ReplaceVarInString(buf, "VERSION", buf, VERSION);
				ReplaceVarInString(buf, "HUBNAME", buf, server.mC.hub_name);
				ReplaceVarInString(buf, "VERSION_DATE", buf, __CURR_DATE_TIME__);
				ReplaceVarInString(buf, "NICK", buf, conn->mpUser->mNick);
				ReplaceVarInString(buf, "SHARE", buf, conn->mpUser->mShare);
				ReplaceVarInString(buf, "TOTAL_SHARE", buf,server.mTotalShare);
			}
			// @CHANGED by dReiska +BEGINS+
			if (mFlags & eTF_SENDTOALL) {
				if (!(mFlags & eTF_SENDPM)) {
					if (!mSendAs.size()) server.DCPublicHSToAll(buf);
					else server.DCPublicToAll(mSendAs,buf);
				} else {/*
					if (!mSendAs.size()) server.DCPrivateHSToAll(buf);
					else server.DCPrivateToAll(mSendAs,buf);
					*/
				}
			} else {
				if (!(mFlags & eTF_SENDPM)) {
					if (!mSendAs.size()) server.DCPublicHS(buf, conn);
					else server.DCPublic(mSendAs, buf, conn);
				} else {
					if (!mSendAs.size()) server.DCPrivateHS(buf, conn);
					else server.DCPrivateHS(buf, conn, &mSendAs);
				}
			}
			// @CHANGED by dReiska -ENDS-
			return 1;
		}
	}
	return 0;
}

void cTrigger::OnLoad()
{}

ostream &operator << (ostream &os, cTrigger &tr)
{
	os << tr.mCommand << " " << tr.mDefinition.substr(0,64) << " Flags:" << tr.mFlags << " SendAs:" << tr.mSendAs << " Class: " << tr.mMinClass << ".." << tr.mMaxClass;
	return os;
}

};
};
